@@ -6,8 +6,8 @@ from .model import Channel, Message, MessageReference, AllowedMentions, Snowflak
     Emoji, User, Interaction, InteractionResponse, Webhook, Guild, ApplicationCommand, Invite, Application, FollowedChannel, \
     ThreadMember, ListThreadsResponse, Component, Role, ApplicationCommandOption, GuildApplicationCommandPermissions, \
     ApplicationCommandPermissions, VerificationLevel, DefaultMessageNotificationLevel, ExplicitContentFilterLevel, \
-    SystemChannelFlags, GuildPreview, ChannelTypes, GuildMember, Ban, PermissionFlags, PruneCountResponse, FILE_TYPE, \
-    VoiceRegion, Integration, ApplicationCommandTypes
+    SystemChannelFlags, GuildPreview, ChannelTypes, GuildMember, Ban, PermissionFlags, GuildWidget, FILE_TYPE, \
+    VoiceRegion, Integration, ApplicationCommandTypes, WelcomeScreen, WelcomeScreenChannel
 from .utils import from_emoji, wrap_to_async
 
 
@@ -15,9 +15,6 @@ class APIClient:
     """
     REST API handling client.
 
-    .. note::
-        If you chose to use async request handler, all request functions will return coroutine which will return raw instance.
-
     :param token: Token of the client.
     :param base: HTTP request handler to use. Must inherit :class:`.base.http.HTTPRequestBase`.
     :param default_allowed_mentions: Default :class:`.model.channel.AllowedMentions` object to use. Default None.
@@ -53,16 +50,16 @@ class APIClient:
                              *,
                              name: str = None,
                              channel_type: int = None,
-                             position: int = None,
-                             topic: str = None,
-                             nsfw: bool = None,
-                             rate_limit_per_user: int = None,
-                             bitrate: int = None,
-                             user_limit: int = None,
-                             permission_overwrites: typing.List[Overwrite] = None,
-                             parent: Channel.TYPING = None,
-                             rtc_region: str = None,
-                             video_quality_mode: int = None,
+                             position: int = EmptyObject,
+                             topic: str = EmptyObject,
+                             nsfw: bool = EmptyObject,
+                             rate_limit_per_user: int = EmptyObject,
+                             bitrate: int = EmptyObject,
+                             user_limit: int = EmptyObject,
+                             permission_overwrites: typing.List[Overwrite] = EmptyObject,
+                             parent: Channel.TYPING = EmptyObject,
+                             rtc_region: str = EmptyObject,
+                             video_quality_mode: int = EmptyObject,
                              reason: str = None):
         if permission_overwrites:
             permission_overwrites = [x.to_dict() for x in permission_overwrites]
@@ -86,7 +83,7 @@ class APIClient:
                               archived: bool = None,
                               auto_archive_duration: int = None,
                               locked: bool = None,
-                              rate_limit_per_user: int = None,
+                              rate_limit_per_user: int = EmptyObject,
                               reason: str = None):
         channel = self.http.modify_thread_channel(int(channel), name, archived, auto_archive_duration, locked, rate_limit_per_user, reason=reason)
         if isinstance(channel, dict):
@@ -167,7 +164,7 @@ class APIClient:
         if embed:
             embeds = [embed]
         if embeds:
-            embeds = [x.to_dict() for x in embeds if not isinstance(x, dict)]
+            embeds = [x if isinstance(x, dict) else x.to_dict() for x in embeds]
         if message_reference and not isinstance(message_reference, dict):
             message_reference = message_reference.to_dict()
         if component and components:
@@ -269,7 +266,7 @@ class APIClient:
             if embed:
                 embeds = [embed]
             if embeds:
-                embeds = [x.to_dict() for x in embeds if not isinstance(x, dict)]
+                embeds = [x if isinstance(x, dict) else x.to_dict() for x in embeds]
         _att = []
         if attachments:
             for x in attachments:
@@ -828,6 +825,7 @@ class APIClient:
     def modify_guild_role(self,
                           guild: Guild.TYPING,
                           role: Role.TYPING,
+                          *,
                           name: str = EmptyObject,
                           permissions: typing.Union[int, str, PermissionFlags] = EmptyObject,
                           color: int = EmptyObject,
@@ -854,12 +852,13 @@ class APIClient:
         return self.http.delete_guild_role(int(guild), int(role), reason=reason)
 
     def request_guild_prune_count(self, guild: Guild.TYPING, *, days: int = None, include_roles: typing.List[Role.TYPING] = None):
+        from .base.model import AbstractObject
         if include_roles is not None:
             include_roles = [*map(str, include_roles)]
         resp = self.http.request_guild_prune_count(int(guild), days, include_roles)
         if isinstance(resp, dict):
-            return PruneCountResponse(resp)
-        return wrap_to_async(PruneCountResponse, None, resp, as_create=False)
+            return AbstractObject(resp)
+        return wrap_to_async(AbstractObject, None, resp, as_create=False)
 
     def begin_guild_prune(self,
                           guild: Guild.TYPING,
@@ -868,12 +867,13 @@ class APIClient:
                           compute_prune_count: bool = True,
                           include_roles: typing.List[Role.TYPING] = None,
                           reason: str = None):
+        from .base.model import AbstractObject
         if include_roles is not None:
             include_roles = [*map(str, include_roles)]
         resp = self.http.begin_guild_prune(int(guild), days, compute_prune_count, include_roles, reason=reason)
         if isinstance(resp, dict):
-            return PruneCountResponse(resp)
-        return wrap_to_async(PruneCountResponse, None, resp, as_create=False)
+            return AbstractObject(resp)
+        return wrap_to_async(AbstractObject, None, resp, as_create=False)
 
     def request_guild_voice_regions(self, guild: Guild.TYPING):
         resp = self.http.request_guild_voice_regions(int(guild))
@@ -896,6 +896,82 @@ class APIClient:
     def delete_guild_integration(self, guild: Guild.TYPING, integration: Integration.TYPING, *, reason: str = None):
         return self.http.delete_guild_integration(int(guild), int(integration), reason=reason)
 
+    def request_guild_widget_settings(self, guild: Guild.TYPING):
+        resp = self.http.request_guild_widget_settings(int(guild))
+        if isinstance(resp, dict):
+            return GuildWidget(resp)
+        return wrap_to_async(GuildWidget, None, resp, as_create=False)
+
+    def modify_guild_widget(self, guild: Guild.TYPING, *, enabled: bool = None, channel: Channel.TYPING = EmptyObject, reason: str = None):
+        resp = self.http.modify_guild_widget(int(guild), enabled, channel, reason=reason)
+        if isinstance(resp, dict):
+            return GuildWidget(resp)
+        return wrap_to_async(GuildWidget, None, resp, as_create=False)
+
+    def request_guild_widget(self, guild: Guild.TYPING):
+        from .base.model import AbstractObject
+        resp = self.http.request_guild_widget(int(guild))
+        if isinstance(resp, dict):
+            return AbstractObject(resp)
+        return wrap_to_async(AbstractObject, None, resp, as_create=False)
+
+    def request_guild_vanity_url(self, guild: Guild.TYPING):
+        from .base.model import AbstractObject
+        resp = self.http.request_guild_vanity_url(int(guild))
+        if isinstance(resp, dict):
+            return AbstractObject(resp)
+        return wrap_to_async(AbstractObject, None, resp, as_create=False)
+
+    def request_guild_widget_image(self, guild: Guild.TYPING, style: str = None):
+        return self.http.request_guild_widget_image(int(guild), style)
+
+    def request_guild_welcome_screen(self, guild: Guild.TYPING):
+        resp = self.http.request_guild_welcome_screen(int(guild))
+        if isinstance(resp, dict):
+            return WelcomeScreen(resp)
+        return wrap_to_async(WelcomeScreen, None, resp, as_create=False)
+
+    def modify_guild_welcome_screen(self,
+                                    guild: Guild.TYPING,
+                                    *,
+                                    enabled: bool = EmptyObject,
+                                    welcome_channels: typing.List[typing.Union[WelcomeScreenChannel, dict]] = EmptyObject,
+                                    description: str = EmptyObject,
+                                    reason: str = None):
+        if welcome_channels is not EmptyObject:
+            welcome_channels = [x if isinstance(x, dict) else x.to_dict() for x in welcome_channels or []]
+        resp = self.http.modify_guild_welcome_screen(int(guild), enabled, welcome_channels, description, reason=reason)
+        if isinstance(resp, dict):
+            return WelcomeScreen(resp)
+        return wrap_to_async(WelcomeScreen, None, resp, as_create=False)
+
+    def modify_user_voice_state(self,
+                                guild: Guild.TYPING,
+                                channel: Channel.TYPING,
+                                user: User.TYPING = "@me",
+                                *,
+                                suppress: bool = None,
+                                request_to_speak_timestamp: typing.Union[datetime.datetime, str] = None):
+        user = int(user) if user != "@me" else user
+        if request_to_speak_timestamp is not None:
+            request_to_speak_timestamp = request_to_speak_timestamp if isinstance(request_to_speak_timestamp, str) else \
+                request_to_speak_timestamp.isoformat()
+        return self.http.modify_user_voice_state(int(guild), str(int(channel)), user, suppress, request_to_speak_timestamp)
+
+    # Invite Requests
+
+    def request_invite(self, invite_code: typing.Union[str, Invite], *, with_counts: bool = None, with_expiration: bool = None):
+        resp = self.http.request_invite(str(invite_code), with_counts, with_expiration)
+        if isinstance(resp, dict):
+            return Invite(self, resp)
+        return wrap_to_async(Invite, self, resp, as_create=False)
+
+    def delete_invite(self, invite_code, *, reason: str = None):
+        resp = self.http.delete_invite(str(invite_code), reason=reason)
+        if isinstance(resp, dict):
+            return Invite(self, resp)
+        return wrap_to_async(Invite, self, resp, as_create=False)
+
     # Webhook
 
     def create_webhook(self, channel: Channel.TYPING, *, name: str = None, avatar: str = None):
@@ -1047,7 +1123,7 @@ class APIClient:
             if embed:
                 embeds = [embed]
             if embeds:
-                embeds = [x.to_dict() for x in embeds if not isinstance(x, dict)]
+                embeds = [x if isinstance(x, dict) else x.to_dict() for x in embeds]
         if component is None or components is None:
             components = None
         else:
@@ -1299,7 +1375,7 @@ class APIClient:
             if embed:
                 embeds = [embed]
             if embeds:
-                embeds = [x.to_dict() for x in embeds if not isinstance(x, dict)]
+                embeds = [x if isinstance(x, dict) else x.to_dict() for x in embeds]
         if component is None or components is None:
             components = None
         else:
