@@ -62,16 +62,16 @@ class HTTPRequestBase(ABC):
                              channel_id,
                              name: str = None,
                              channel_type: int = None,
-                             position: int = None,
-                             topic: str = None,
-                             nsfw: bool = None,
-                             rate_limit_per_user: int = None,
-                             bitrate: int = None,
-                             user_limit: int = None,
-                             permission_overwrites: typing.List[dict] = None,
-                             parent_id: str = None,
-                             rtc_region: str = None,
-                             video_quality_mode: int = None,
+                             position: int = EmptyObject,
+                             topic: str = EmptyObject,
+                             nsfw: bool = EmptyObject,
+                             rate_limit_per_user: int = EmptyObject,
+                             bitrate: int = EmptyObject,
+                             user_limit: int = EmptyObject,
+                             permission_overwrites: typing.List[dict] = EmptyObject,
+                             parent_id: str = EmptyObject,
+                             rtc_region: str = EmptyObject,
+                             video_quality_mode: int = EmptyObject,
                              reason: str = None):
         """
         Sends modify channel request, for guild channel.
@@ -96,25 +96,25 @@ class HTTPRequestBase(ABC):
             body["name"] = name
         if channel_type is not None:
             body["channel_type"] = channel_type
-        if position is not None:
+        if position is not EmptyObject:
             body["position"] = position
-        if topic is not None:
+        if topic is not EmptyObject:
             body["topic"] = topic
-        if nsfw is not None:
+        if nsfw is not EmptyObject:
             body["nsfw"] = nsfw
-        if rate_limit_per_user is not None:
+        if rate_limit_per_user is not EmptyObject:
             body["rate_limit_per_user"] = rate_limit_per_user
-        if bitrate is not None:
+        if bitrate is not EmptyObject:
             body["bitrate"] = bitrate
-        if user_limit is not None:
+        if user_limit is not EmptyObject:
             body["user_limit"] = user_limit
-        if permission_overwrites is not None:
+        if permission_overwrites is not EmptyObject:
             body["permission_overwrites"] = permission_overwrites
-        if parent_id is not None:
+        if parent_id is not EmptyObject:
             body["parent_id"] = parent_id
-        if rtc_region is not None:
+        if rtc_region is not EmptyObject:
             body["rtc_region"] = rtc_region
-        if video_quality_mode is not None:
+        if video_quality_mode is not EmptyObject:
             body["video_quality_mode"] = video_quality_mode
         return self.request(f"/channels/{channel_id}", "PATCH", body, is_json=True, reason_header=reason)
 
@@ -140,7 +140,7 @@ class HTTPRequestBase(ABC):
                               archived: bool = None,
                               auto_archive_duration: int = None,
                               locked: bool = None,
-                              rate_limit_per_user: int = None,
+                              rate_limit_per_user: int = EmptyObject,
                               reason: str = None):
         """
         Sends modify channel request, for thread channel.
@@ -162,7 +162,7 @@ class HTTPRequestBase(ABC):
             body["auto_archive_duration"] = auto_archive_duration
         if locked is not None:
             body["locked"] = locked
-        if rate_limit_per_user is not None:
+        if rate_limit_per_user is not EmptyObject:
             body["rate_limit_per_user"] = rate_limit_per_user
         return self.request(f"/channels/{channel_id}", "PATCH", body, is_json=True, reason_header=reason)
 
@@ -1322,6 +1322,132 @@ class HTTPRequestBase(ABC):
         """
         return self.request(f"/guilds/{guild_id}/integrations/{integration_id}", "DELETE", reason_header=reason)
 
+    def request_guild_widget_settings(self, guild_id):
+        """
+        Sends get guild widget settings request.
+
+        :param guild_id: ID of the guild.
+        """
+        return self.request(f"/guilds/{guild_id}/widget", "GET")
+
+    def modify_guild_widget(self, guild_id, enabled: bool = None, channel_id: str = EmptyObject, reason: str = None):
+        """
+        Sends modify guild widget request.
+
+        :param guild_id: ID of the guild.
+        :param enabled: Whether the widget is enabled.
+        :param channel_id: ID of the channel for the widget.
+        :param reason: Reason of the action.
+        """
+        body = {}
+        if enabled is not None:
+            body["enabled"] = enabled
+        if channel_id is not EmptyObject:
+            body["channel_id"] = channel_id
+        return self.request(f"/guilds/{guild_id}/widget", "PATCH", body, is_json=True, reason_header=reason)
+
+    def request_guild_widget(self, guild_id):
+        """
+        Sends get guild widget request.
+
+        :param guild_id: ID of the guild.
+        """
+        return self.request(f"/guilds/{guild_id}/widget.json", "GET")
+
+    def request_guild_vanity_url(self, guild_id):
+        """
+        Sends get guild vanity URL request.
+
+        :param guild_id: ID of the guild.
+        """
+        return self.request(f"/guilds/{guild_id}/vanity-url", "GET")
+
+    def request_guild_widget_image(self, guild_id, style: str = None):
+        """
+        Sends get guild widget image request.
+
+        :param guild_id: ID of the guild.
+        :param style: Style to use. Can be one of ``shield``, ``banner1``, ``banner2``, ``banner3``, or ``banner4``.
+        """
+        params = f"?style={style}" if style is not None else ""
+        return self.download(f"{self.BASE_URL}/guilds/{guild_id}/widget.png{params}")
+
+    def request_guild_welcome_screen(self, guild_id):
+        """
+        Sends get guild welcome screen request.
+
+        :param guild_id: ID of the guild.
+        """
+        return self.request(f"/guilds/{guild_id}/welcome-screen", "GET")
+
+    def modify_guild_welcome_screen(self,
+                                    guild_id, enabled: bool = EmptyObject,
+                                    welcome_channels: typing.List[dict] = EmptyObject,
+                                    description: str = EmptyObject,
+                                    reason: str = None):
+        """
+        Sends modify guild welcome screen request.
+
+        :param guild_id: ID of the guild.
+        :param enabled: Whether it is enabled.
+        :param welcome_channels: Welcome channels to link.
+        :param description: Description to show in welcome screen.
+        :param reason: Reason of the action.
+        """
+        body = {}
+        if enabled is not EmptyObject:
+            body["enabled"] = enabled
+        if welcome_channels is not EmptyObject:
+            body["welcome_channels"] = welcome_channels
+        if description is not EmptyObject:
+            body["description"] = description
+        return self.request(f"/guilds/{guild_id}/welcome-screen", "PATCH", body, is_json=True, reason_header=reason)
+
+    def modify_user_voice_state(self, guild_id, channel_id: str, user_id="@me", suppress: bool = None, request_to_speak_timestamp: str = None):
+        """
+        Sends modify curren user or user voice state requset.
+
+        :param guild_id: ID of the guild.
+        :param channel_id: ID of the stage channel.
+        :param user_id: ID of the user to modify. Set to ``@me`` for current user.
+        :param suppress: Whether suppress is enabled.
+        :param request_to_speak_timestamp: When to set the user to request to speak. Exclusive to current user(``@me``).
+        """
+        if user_id != "@me" and request_to_speak_timestamp is not None:
+            raise TypeError("request_to_speak_timestamp is exclusive to @me.")
+        body = {"channel_id": channel_id}
+        if suppress is not None:
+            body["suppress"] = suppress
+        if request_to_speak_timestamp is not None:
+            body["request_to_speak_timestamp"] = request_to_speak_timestamp
+        return self.request(f"/guilds/{guild_id}/voice-states/{user_id}", "PATCH", body, is_json=True)
+
+    # Invite Requests
+
+    def request_invite(self, invite_code, with_counts: bool = None, with_expiration: bool = None):
+        """
+        Sends get invite request.
+
+        :param invite_code: Code of the invite.
+        :param with_counts: Whether to include counts.
+        :param with_expiration: Whether to include expiration info.
+        """
+        params = {}
+        if with_counts is not None:
+            params["with_counts"] = with_counts
+        if with_expiration is not None:
+            params["with_expiration"] = with_expiration
+        return self.request(f"/invites/{invite_code}", "GET", params=params)
+
+    def delete_invite(self, invite_code, reason: str = None):
+        """
+        Sends delete invite request.
+
+        :param invite_code: Code of the invite.
+        :param reason: Reason of the action.
+        """
+        return self.request(f"/invites/{invite_code}", "DELETE", reason_header=reason)
+
     # Webhook Requests
 
     def create_webhook(self, channel_id, name: str, avatar: str = None):
