@@ -3,6 +3,9 @@ import typing
 
 from ..model.snowflake import Snowflake
 
+if typing.TYPE_CHECKING:
+    from ..api import APIClient
+
 
 class CopyableObject:
     def copy(self):
@@ -26,9 +29,9 @@ class DiscordObjectBase:
     def __init__(self, client, resp, **kwargs):
         resp.update(kwargs)
         self._cache_type = None
-        self.raw = resp
-        self.id = Snowflake(resp["id"])
-        self.client = client
+        self.raw: dict = resp
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.client: APIClient = client
 
     def __int__(self):
         return int(self.id)
@@ -52,6 +55,17 @@ class DiscordObjectBase:
             return ret
 
 
+class AbstractObject(dict):
+    def __init__(self, resp):
+        super().__init__(**resp)
+
+    def __getattr__(self, item):
+        return self.get(item)
+
+    def __setattr__(self, key, value):
+        self[key] = value
+
+
 class FlagBase:
     def __init__(self, *args, **kwargs):
         self.values = {x: getattr(self, x) for x in dir(self) if isinstance(getattr(self, x), int)}
