@@ -7,9 +7,15 @@ from .emoji import Emoji
 from .guild import GuildMember
 from .permission import PermissionFlags, Role
 from .snowflake import Snowflake
+from .sticker import Sticker, StickerItem
 from .user import User
 from ..base.model import CopyableObject, DiscordObjectBase, TypeBase, FlagBase
 
+if typing.TYPE_CHECKING:
+    from .gateway import Application
+    from .guild import Guild
+    from ..api import APIClient
+
 
 class Channel(DiscordObjectBase):
     TYPING = typing.Union[int, str, Snowflake, "Channel"]
@@ -17,33 +23,34 @@ class Channel(DiscordObjectBase):
     def __init__(self, client, resp, *, guild_id=None):
         super().__init__(client, resp)
         self._cache_type = "channel"
-        self.type = ChannelTypes(resp["type"])
-        self.guild_id = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
-        self.position = resp.get("position")
-        self.permission_overwrites = [Overwrite.create(x) for x in resp.get("permission_overwrites", [])]
-        self.name = resp.get("name")
-        self.topic = resp.get("topic")
-        self.nsfw = resp.get("nsfw")
-        self.last_message_id = Snowflake.optional(resp.get("last_message_id"))
-        self.bitrate = resp.get("bitrate")
-        self.user_limit = resp.get("user_limit")
-        self.rate_limit_per_user = resp.get("rate_limit_per_user")
-        self.recipients = resp.get("recipients", [])
-        self.icon = resp.get("icon")
-        self.owner_id = Snowflake.optional(resp.get("owner_id"))
-        self.application_id = Snowflake.optional(resp.get("application_id"))
-        self.parent_id = Snowflake.optional(resp.get("parent_id"))
+        self.type: ChannelTypes = ChannelTypes(resp["type"])
+        self.guild_id: Snowflake = Snowflake.optional(resp.get("guild_id")) or Snowflake.ensure_snowflake(guild_id)
+        self.position: typing.Optional[int] = resp.get("position")
+        self.permission_overwrites: typing.Optional[typing.List[Overwrite]] = [Overwrite.create(x) for x in resp.get("permission_overwrites", [])]
+        self.name: typing.Optional[str] = resp.get("name")
+        self.topic: typing.Optional[str] = resp.get("topic")
+        self.nsfw: typing.Optional[bool] = resp.get("nsfw")
+        self.last_message_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("last_message_id"))
+        self.bitrate: typing.Optional[int] = resp.get("bitrate")
+        self.user_limit: typing.Optional[int] = resp.get("user_limit")
+        self.rate_limit_per_user: typing.Optional[int] = resp.get("rate_limit_per_user")
+        self.recipients: typing.Optional[typing.List[User]] = [User.create(client, x) for x in resp.get("recipients", [])]
+        self.icon: typing.Optional[str] = resp.get("icon")
+        self.owner_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("owner_id"))
+        self.application_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("application_id"))
+        self.parent_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("parent_id"))
         self.__last_pin_timestamp = resp.get("last_pin_timestamp")
-        self.last_pin_timestamp = datetime.datetime.fromisoformat(self.__last_pin_timestamp) if self.__last_pin_timestamp else self.__last_pin_timestamp
-        self.rtc_region = resp.get("rtc_region")
+        self.last_pin_timestamp: typing.Optional[datetime.datetime] = datetime.datetime.fromisoformat(self.__last_pin_timestamp) if self.__last_pin_timestamp else self.__last_pin_timestamp
+        self.rtc_region: typing.Optional[str] = resp.get("rtc_region")
         self.__video_quality_mode = resp.get("video_quality_mode")
-        self.video_quality_mode = VideoQualityModes(self.__video_quality_mode) if self.__video_quality_mode else self.__video_quality_mode
-        self.message_count = resp.get("message_count")
-        self.member_count = resp.get("member_count")
-        self.thread_metadata = ThreadMetadata.optional(self.client, resp.get("thread_metadata"))
-        self.member = ThreadMember.optional(self.client, resp.get("member"))
-        self.default_auto_archive_duration = resp.get("default_auto_archive_duration")
-        self.permissions = resp.get("permissions")
+        self.video_quality_mode: typing.Optional[VideoQualityModes] = VideoQualityModes(self.__video_quality_mode) if self.__video_quality_mode else self.__video_quality_mode
+        self.message_count: typing.Optional[int] = resp.get("message_count")
+        self.member_count: typing.Optional[int] = resp.get("member_count")
+        self.thread_metadata: typing.Optional[ThreadMetadata] = ThreadMetadata.optional(self.client, resp.get("thread_metadata"))
+        self.member: typing.Optional[ThreadMember] = ThreadMember.optional(self.client, resp.get("member"))
+        self.default_auto_archive_duration: typing.Optional[int] = resp.get("default_auto_archive_duration")
+        self.__permissions = resp.get("permissions")
+        self.permissions: typing.Optional[PermissionFlags] = PermissionFlags.from_value(int(self.__permissions)) if self.__permissions else self.__permissions
 
     def modify(self, **kwargs):
         if self.type.group_dm:
@@ -62,7 +69,7 @@ class Channel(DiscordObjectBase):
     def delete(self, *, reason: str = None):
         return self.client.delete_channel(self, reason=reason)
 
-    def create_message(self, *args, **kwargs):
+    def create_message(self, *args, **kwargs) -> typing.Union["Message", typing.Awaitable["Message"]]:
         if not self.is_messageable():
             raise TypeError("You can't send message in this type of channel.")
         return self.client.create_message(self, *args, **kwargs)
@@ -71,7 +78,7 @@ class Channel(DiscordObjectBase):
     def send(self):
         return self.create_message
 
-    def bulk_delete_messages(self, *messages: typing.Union[int, str, Snowflake, "Message"], reason: str = None):
+    def bulk_delete_messages(self, *messages: "Message.TYPING", reason: str = None):
         return self.client.bulk_delete_messages(self, *messages, reason=reason)
 
     def edit_permissions(self, overwrite, *, reason: str = None):
@@ -86,7 +93,7 @@ class Channel(DiscordObjectBase):
     def delete_permissions(self, overwrite, *, reason: str = None):
         return self.client.delete_channel_permission(self, overwrite, reason=reason)
 
-    def follow(self, target_channel: typing.Union[int, str, Snowflake, "Channel"]):
+    def follow(self, target_channel: "Channel.TYPING"):
         return self.client.follow_news_channel(self, target_channel)
 
     def trigger_typing_indicator(self):
@@ -95,17 +102,17 @@ class Channel(DiscordObjectBase):
     def request_pinned_messages(self):
         return self.client.request_pinned_messages(self)
 
-    def add_recipient(self, user: typing.Union[int, str, Snowflake, User], access_token: str, nick: str):
+    def add_recipient(self, user: User.TYPING, access_token: str, nick: str):
         if not self.type.group_dm:
             raise AttributeError("This type of channel is not allowed to add recipient.")
         return self.client.group_dm_add_recipient(self, user, access_token, nick)
 
-    def remove_recipient(self, user: typing.Union[int, str, Snowflake, User]):
+    def remove_recipient(self, user: User.TYPING):
         if not self.type.group_dm:
             raise AttributeError("This type of channel is not allowed to remove recipient.")
         return self.client.group_dm_remove_recipient(self, user)
 
-    def start_thread(self, message: typing.Union[int, str, Snowflake, "Message"] = None, *, name: str, auto_archive_duration: int, reason: str = None):
+    def start_thread(self, message: "Message.TYPING" = None, *, name: str, auto_archive_duration: int, reason: str = None):
         return self.client.start_thread(self, message, name=name, auto_archive_duration=auto_archive_duration, reason=reason)
 
     def join_thread(self):
@@ -113,17 +120,17 @@ class Channel(DiscordObjectBase):
             raise AttributeError("This type of channel is not allowed to join thread.")
         return self.client.join_thread(self)
 
-    def add_thread_member(self, user: typing.Union[int, str, Snowflake, User]):
+    def add_thread_member(self, user: User.TYPING):
         if not self.is_thread_channel():
             raise AttributeError("This type of channel is not allowed to add thread member.")
-        return self.client.add_thread_member(user)
+        return self.client.add_thread_member(self, user)
 
     def leave_thread(self):
         if not self.is_thread_channel():
             raise AttributeError("This type of channel is not allowed to leave thread.")
         return self.client.leave_thread(self)
 
-    def remove_thread_member(self, user: typing.Union[int, str, Snowflake, User]):
+    def remove_thread_member(self, user: User.TYPING):
         if not self.is_thread_channel():
             raise AttributeError("This type of channel is not allowed to remove thread member.")
         return self.client.remove_thread_member(self, user)
@@ -137,20 +144,20 @@ class Channel(DiscordObjectBase):
         return self.client.list_active_threads(self)
 
     def list_public_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
-        return self.client.list_public_archived_threads(self, before, limit)
+        return self.client.list_public_archived_threads(self, before=before, limit=limit)
 
     def list_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
-        return self.client.list_private_archived_threads(self, before, limit)
+        return self.client.list_private_archived_threads(self, before=before, limit=limit)
 
     def list_joined_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
-        return self.client.list_joined_private_archived_threads(self, before, limit)
+        return self.client.list_joined_private_archived_threads(self, before=before, limit=limit)
 
     def archive(self, locked: bool = False):
         if not self.is_thread_channel():
             raise AttributeError("This type of channel is not allowed to archive.")
         return self.modify(archived=True, locked=locked)
 
-    def to_position_param(self, position: int = None, lock_permissions: bool = None, parent: typing.Union[int, str, Snowflake, "Channel"] = None):
+    def to_position_param(self, position: int = None, lock_permissions: bool = None, parent: typing.Union[int, str, Snowflake, "Channel"] = None) -> dict:
         if isinstance(parent, Channel) and not parent.type.guild_category:
             raise TypeError("parent must be category channel.")
         param = {
@@ -162,18 +169,20 @@ class Channel(DiscordObjectBase):
         return param
 
     @property
-    def mention(self):
+    def mention(self) -> str:
         return f"<#{self.id}>"
 
     @property
-    def guild(self):
+    def guild(self) -> typing.Optional["Guild"]:
         if self.guild_id and self.client.has_cache:
-            return self.client.cache.get(self.guild_id, "guild")
+            return self.client.cache.get(self.guild_id, "guild")  # noqa
 
-    def is_messageable(self):
+    def is_messageable(self) -> bool:
+        if self.is_thread_channel():
+            return self.thread_metadata.archived
         return self.type.guild_text or self.type.guild_news or self.type.dm or self.type.group_dm
 
-    def is_thread_channel(self):
+    def is_thread_channel(self) -> bool:
         return self.type.guild_news_thread or self.type.guild_public_thread or self.type.guild_private_thread
 
 
@@ -203,46 +212,50 @@ class Message(DiscordObjectBase):
         from .interactions import MessageInteraction, Component  # Prevent circular import.
         super().__init__(client, resp)
         self._cache_type = "message"
-        self.channel_id = Snowflake(resp["channel_id"])
-        self.guild_id = Snowflake.optional(resp.get("guild_id") or guild_id)
-        self.author = User.create(client, resp["author"])
+        self.channel_id: Snowflake = Snowflake(resp["channel_id"])
+        self.guild_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("guild_id") or guild_id)
+        self.author: User = User.create(client, resp["author"])
         self.__member = resp.get("member")
-        self.member = GuildMember.create(self.client, self.__member, user=self.author, guild_id=self.guild_id) if self.__member else self.__member
-        self.content = resp["content"]
-        self.timestamp = datetime.datetime.fromisoformat(resp["timestamp"])
+        self.member: GuildMember = GuildMember.create(self.client, self.__member, user=self.author, guild_id=self.guild_id) if self.__member else self.__member
+        self.content: str = resp["content"]
+        self.timestamp: datetime.datetime = datetime.datetime.fromisoformat(resp["timestamp"])
         self.__edited_timestamp = resp["edited_timestamp"]
-        self.edited_timestamp = datetime.datetime.fromisoformat(self.__edited_timestamp) if self.__edited_timestamp else self.__edited_timestamp
-        self.tts = resp["tts"]
-        self.mention_everyone = resp["mention_everyone"]
-        self.mentions = [GuildMember.create(self.client, x["member"], user=User.create(self.client, x), guild_id=self.guild_id)
-                         if "member" in x else User.create(self.client, x) for x in resp["mentions"]]
-        self.mention_roles = [Snowflake(x) for x in resp["mention_roles"]]
-        self.mention_channels = [ChannelMention(x) for x in resp.get("mention_channels", [])]
-        self.attachments = [Attachment(self.client, x) for x in resp["attachments"] or []]
-        self.embeds = [Embed.create(x) for x in resp["embeds"] or []]
-        self.reactions = [Reaction(self.client, x) for x in resp.get("reactions", [])]
-        self.nonce = resp.get("nonce")
-        self.pinned = resp["pinned"]
-        self.webhook_id = Snowflake.optional(resp.get("webhook_id"))
+        self.edited_timestamp: typing.Optional[datetime.datetime] = datetime.datetime.fromisoformat(self.__edited_timestamp) if self.__edited_timestamp else self.__edited_timestamp
+        self.tts: bool = resp["tts"]
+        self.mention_everyone: bool = resp["mention_everyone"]
+        self.mentions: typing.List[typing.Union[User, GuildMember]] = [GuildMember.create(self.client, x["member"], user=User.create(self.client, x), guild_id=self.guild_id)
+                                                                       if "member" in x else User.create(self.client, x) for x in resp["mentions"]]
+        self.mention_roles: typing.List[Snowflake] = [Snowflake(x) for x in resp["mention_roles"]]
+        self.mention_channels: typing.List[ChannelMention] = [ChannelMention(x) for x in resp.get("mention_channels", [])]
+        self.attachments: typing.List[Attachment] = [Attachment(self.client, x) for x in resp["attachments"] or []]
+        self.embeds: typing.List[Embed] = [Embed.create(x) for x in resp["embeds"] or []]
+        self.reactions: typing.Optional[typing.List[Reaction]] = [Reaction(self.client, x) for x in resp.get("reactions", [])]
+        self.nonce: typing.Optional[typing.Union[int, str]] = resp.get("nonce")
+        self.pinned: bool = resp["pinned"]
+        self.webhook_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("webhook_id"))
         self.__webhook_token = webhook_token
         self.__interaction_token = interaction_token
         self.__original_response = original_response
-        self.type = MessageTypes(resp["type"])
-        self.activity = MessageActivity.optional(resp.get("activity"))
-        self.application = resp.get("application")
-        self.message_reference = MessageReference(resp.get("message_reference", {}))
+        self.type: MessageTypes = MessageTypes(resp["type"])
+        self.activity: typing.Optional[MessageActivity] = MessageActivity.optional(resp.get("activity"))
+        self.application: typing.Optional[Application] = resp.get("application")
+        self.application_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("application_id"))
+        self.message_reference: typing.Optional[MessageReference] = MessageReference(resp.get("message_reference", {}))
         self.__flags = resp.get("flags")
-        self.flags = MessageFlags.from_value(self.__flags) if self.__flags else self.__flags
-        self.stickers = [MessageSticker(x) for x in resp.get("stickers", [])]
+        self.flags: MessageFlags = MessageFlags.from_value(self.__flags) if self.__flags else self.__flags
         self.__referenced_message = resp.get("referenced_message")
-        self.referenced_message = Message.create(self.client, self.__referenced_message, guild_id=self.guild_id) if self.__referenced_message else self.__referenced_message
+        self.referenced_message: typing.Optional[Message] = Message.create(self.client, self.__referenced_message, guild_id=self.guild_id) if self.__referenced_message else self.__referenced_message
         self.__interaction = resp.get("interaction")
-        self.interaction = MessageInteraction(self.client, self.__interaction) if self.__interaction else self.__interaction
+        self.interaction: typing.Optional[MessageInteraction] = MessageInteraction(self.client, self.__interaction) if self.__interaction else self.__interaction
         self.__thread = resp.get("thread")
-        self.thread = Channel.create(self.client, self.__thread, guild_id=self.guild_id, ensure_cache_type="channel") if self.__thread else self.__thread
-        self.components = [Component.auto_detect(x) for x in resp.get("components", [])]
+        self.thread: typing.Optional[Channel] = Channel.create(self.client, self.__thread, guild_id=self.guild_id, ensure_cache_type="channel") if self.__thread else self.__thread
+        self.components: typing.Optional[typing.List[Component]] = [Component.auto_detect(x) for x in resp.get("components", [])]
+        self.sticker_items: typing.Optional[typing.List[StickerItem]] = [StickerItem(x) for x in resp.get("sticker_items", [])]
+        self.stickers: typing.Optional[typing.List[Sticker]] = [Sticker.create(client, x) for x in resp.get("stickers", [])]
 
-    def reply(self, content=None, **kwargs):
+        # self.stickers: typing.Optional[typing.List[MessageSticker]] = [MessageSticker(x) for x in resp.get("stickers", [])]
+
+    def reply(self, content=None, **kwargs) -> typing.Union["Message", typing.Awaitable["Message"]]:
         kwargs["message_reference"] = self
         mention = kwargs.pop("mention") if "mention" in kwargs.keys() else True
         allowed_mentions = kwargs.get("allowed_mentions", self.client.default_allowed_mentions or AllowedMentions(replied_user=mention)).copy()
@@ -274,7 +287,7 @@ class Message(DiscordObjectBase):
     def create_reaction(self, emoji: typing.Union[Emoji, str]):
         return self.client.create_reaction(self.channel_id, self.id, emoji)
 
-    def delete_reaction(self, emoji: typing.Union[Emoji, str], user: typing.Union[int, str, Snowflake, User] = "@me"):
+    def delete_reaction(self, emoji: typing.Union[Emoji, str], user: User.TYPING = "@me"):
         return self.client.delete_reaction(self.channel_id, self.id, emoji, user)
 
     def pin(self, *, reason: str = None):
@@ -287,12 +300,12 @@ class Message(DiscordObjectBase):
         return self.client.start_thread(self.channel_id, self, name=name, auto_archive_duration=auto_archive_duration, reason=reason)
 
     @property
-    def guild(self):
+    def guild(self) -> typing.Optional["Guild"]:
         if self.guild_id and self.client.has_cache:
             return self.client.get(self.guild_id, "guild")
 
     @property
-    def channel(self):
+    def channel(self) -> typing.Optional[Channel]:
         if self.channel_id and self.client.has_cache:
             if self.guild_id:
                 return self.guild.get(self.channel_id, "channel") or self.client.get(self.channel_id, "channel")
@@ -327,8 +340,8 @@ class MessageTypes(TypeBase):
 
 class MessageActivity:
     def __init__(self, resp):
-        self.type = MessageActivityTypes(resp["type"])
-        self.party_id = resp.get("party_id")  # This is actually set as string in discord docs.
+        self.type: MessageActivityTypes = MessageActivityTypes(resp["type"])
+        self.party_id: str = resp.get("party_id")  # This is actually set as string in discord docs.
 
     @classmethod
     def optional(cls, resp):
@@ -354,31 +367,14 @@ class MessageFlags(FlagBase):
     LOADING = 1 << 7
 
 
-class MessageSticker:
-    def __init__(self, resp):
-        self.id = Snowflake(resp["id"])
-        self.pack_id = Snowflake(resp["pack_id"])
-        self.name = resp["name"]
-        self.description = resp["description"]
-        self.tags = resp.get("tags")
-        self.asset = resp["asset"]
-        self.format_type = MessageStickerFormatTypes(resp["format_type"])
-
-
-class MessageStickerFormatTypes(TypeBase):
-    PNG = 1
-    APNG = 2
-    LOTTIE = 3
-
-
 class MessageReference:
     def __init__(self, resp):
-        self.message_id = Snowflake.optional(resp.get("message_id"))
-        self.channel_id = Snowflake.optional(resp.get("channel_id"))
-        self.guild_id = Snowflake.optional(resp.get("guild_id"))
-        self.fail_if_not_exists = resp.get("fail_if_not_exists", True)
+        self.message_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("message_id"))
+        self.channel_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("channel_id"))
+        self.guild_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("guild_id"))
+        self.fail_if_not_exists: bool = resp.get("fail_if_not_exists", True)
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         resp = {}
         if self.message_id:
             resp["message_id"] = str(self.message_id)
@@ -400,21 +396,21 @@ class MessageReference:
 
 class FollowedChannel:
     def __init__(self, client, resp):
-        self.client = client
-        self.channel_id = Snowflake(resp["channel_id"])
-        self.webhook_id = Snowflake(resp["webhook_id"])
+        self.client: "APIClient" = client
+        self.channel_id: typing.Optional[Snowflake] = Snowflake(resp["channel_id"])
+        self.webhook_id: typing.Optional[Snowflake] = Snowflake(resp["webhook_id"])
 
     @property
-    def channel(self):
+    def channel(self) -> typing.Optional[Channel]:
         if self.client.has_cache:
             return self.client.get(self.channel_id)
 
 
 class Reaction:
     def __init__(self, client, resp):
-        self.count = resp["count"]
-        self.me = resp["me"]
-        self.emoji = Emoji(client, resp["emoji"])
+        self.count: int = resp["count"]
+        self.me: bool = resp["me"]
+        self.emoji: Emoji = Emoji(client, resp["emoji"])
 
 
 class Overwrite(CopyableObject):
@@ -426,12 +422,12 @@ class Overwrite(CopyableObject):
                  **kw):
         if user is None and role is None:
             raise TypeError("you must pass user or role.")
-        self.id = Snowflake.ensure_snowflake(int(user or role))
-        self.type = 0 if role else 1
-        self.allow = PermissionFlags.from_value(int(allow))
-        self.deny = PermissionFlags.from_value(int(deny))
+        self.id: Snowflake = Snowflake.ensure_snowflake(int(user or role))
+        self.type: int = 0 if role else 1
+        self.allow: PermissionFlags = PermissionFlags.from_value(int(allow))
+        self.deny: PermissionFlags = PermissionFlags.from_value(int(deny))
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         return {"id": str(self.id), "type": self.type, "allow": str(int(self.allow)), "deny": str(int(self.deny))}
 
     def edit(self, **kwargs):
@@ -453,17 +449,13 @@ class Overwrite(CopyableObject):
 
 class ThreadMetadata:
     def __init__(self, client, resp):
-        self.client = client
-        self.archived = resp["archived"]
-        self.archiver_id = Snowflake.optional(resp.get("archiver_id"))
-        self.auto_archive_duration = resp["auto_archive_duration"]
+        self.client: APIClient = client
+        self.archived: bool = resp["archived"]
+        # self.archiver_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("archiver_id"))
+        self.auto_archive_duration: int = resp["auto_archive_duration"]
         self.archive_timestamp = datetime.datetime.fromisoformat(resp["archive_timestamp"])
-        self.locked = resp.get("locked", False)
-
-    @property
-    def archiver(self):
-        if self.client.has_cache and self.archiver_id:
-            return self.client.get(self.archiver_id)
+        self.locked: bool = resp["locked"]
+        self.invitable: typing.Optional[bool] = resp.get("invitable")
 
     @classmethod
     def optional(cls, client, resp):
@@ -473,14 +465,14 @@ class ThreadMetadata:
 
 class ThreadMember:
     def __init__(self, client, resp):
-        self.client = client
-        self.id = Snowflake(resp["id"])
-        self.user_id = Snowflake(resp["user_id"])
-        self.join_timestamp = datetime.datetime.fromisoformat(resp["join_timestamp"])
-        self.flags = resp["flags"]
+        self.client: APIClient = client
+        self.id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("id"))
+        self.user_id: typing.Optional[Snowflake] = Snowflake.optional(resp.get("user_id"))
+        self.join_timestamp: datetime.datetime = datetime.datetime.fromisoformat(resp["join_timestamp"])
+        self.flags: int = resp["flags"]
 
     @property
-    def user(self):
+    def user(self) -> typing.Optional[User]:
         if self.client.has_cache:
             return self.client.get(self.user_id, "user")
 
@@ -499,20 +491,20 @@ class Embed(CopyableObject):
     def __init__(self, *, title: str = None, description: str = None, url: str = None, timestamp: datetime.datetime = None, color: int = None, **kwargs):
         resp = self.__create(title=title, description=description, url=url, timestamp=timestamp, color=color)
         resp.update(kwargs)
-        self.title = resp.get("title")
-        self.type = resp.get("type", "rich")
-        self.description = resp.get("description")
-        self.url = resp.get("url")
+        self.title: typing.Optional[str] = resp.get("title")
+        self.type: typing.Optional[str] = resp.get("type", "rich")
+        self.description: typing.Optional[str] = resp.get("description")
+        self.url: typing.Optional[str] = resp.get("url")
         self.__timestamp = resp.get("timestamp")
-        self.timestamp = datetime.datetime.fromisoformat(self.__timestamp) if self.__timestamp else self.__timestamp
-        self.color = resp.get("color")
-        self.footer = EmbedFooter.optional(resp.get("footer"))
-        self.image = EmbedImage.optional(resp.get("image"))
-        self.thumbnail = EmbedThumbnail.optional(resp.get("thumbnail"))
-        self.video = EmbedVideo.optional(resp.get("video"))
-        self.provider = EmbedProvider.optional(resp.get("provider"))
-        self.author = EmbedAuthor.optional(resp.get("author"))
-        self.fields = [EmbedField(x) for x in resp.get("fields", [])]
+        self.timestamp: typing.Optional[datetime.datetime] = datetime.datetime.fromisoformat(self.__timestamp) if self.__timestamp else self.__timestamp
+        self.color: typing.Optional[int] = resp.get("color")
+        self.footer: typing.Optional[EmbedFooter] = EmbedFooter.optional(resp.get("footer"))
+        self.image: typing.Optional[EmbedImage] = EmbedImage.optional(resp.get("image"))
+        self.thumbnail: typing.Optional[EmbedThumbnail] = EmbedThumbnail.optional(resp.get("thumbnail"))
+        self.video: typing.Optional[EmbedVideo] = EmbedVideo.optional(resp.get("video"))
+        self.provider: typing.Optional[EmbedProvider] = EmbedProvider.optional(resp.get("provider"))
+        self.author: typing.Optional[EmbedAuthor] = EmbedAuthor.optional(resp.get("author"))
+        self.fields: typing.Optional[typing.List[EmbedField]] = [EmbedField(x) for x in resp.get("fields", [])]
 
     @staticmethod
     def __create(*, title: str = None, description: str = None, url: str = None, timestamp: datetime.datetime = None, color: int = None):
@@ -547,7 +539,7 @@ class Embed(CopyableObject):
     def remove_field(self):
         return self.fields.pop
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.title:
             ret["title"] = self.title
@@ -578,12 +570,12 @@ class Embed(CopyableObject):
 
 class EmbedThumbnail(CopyableObject):
     def __init__(self, resp):
-        self.url = resp.get("url")
-        self.proxy_url = resp.get("proxy_url")
-        self.height = resp.get("height")
-        self.width = resp.get("width")
+        self.url: typing.Optional[str] = resp.get("url")
+        self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
+        self.height: typing.Optional[int] = resp.get("height")
+        self.width: typing.Optional[int] = resp.get("width")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.url:
             ret["url"] = self.url
@@ -603,12 +595,12 @@ class EmbedThumbnail(CopyableObject):
 
 class EmbedVideo(CopyableObject):
     def __init__(self, resp):
-        self.url = resp.get("url")
-        self.proxy_url = resp.get("proxy_url")
-        self.height = resp.get("height")
-        self.width = resp.get("width")
+        self.url: typing.Optional[str] = resp.get("url")
+        self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
+        self.height: typing.Optional[int] = resp.get("height")
+        self.width: typing.Optional[int] = resp.get("width")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.url:
             ret["url"] = self.url
@@ -628,12 +620,12 @@ class EmbedVideo(CopyableObject):
 
 class EmbedImage(CopyableObject):
     def __init__(self, resp):
-        self.url = resp.get("url")
-        self.proxy_url = resp.get("proxy_url")
-        self.height = resp.get("height")
-        self.width = resp.get("width")
+        self.url: typing.Optional[str] = resp.get("url")
+        self.proxy_url: typing.Optional[str] = resp.get("proxy_url")
+        self.height: typing.Optional[int] = resp.get("height")
+        self.width: typing.Optional[int] = resp.get("width")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.url:
             ret["url"] = self.url
@@ -653,10 +645,10 @@ class EmbedImage(CopyableObject):
 
 class EmbedProvider(CopyableObject):
     def __init__(self, resp):
-        self.name = resp.get("name")
-        self.url = resp.get("url")
+        self.name: typing.Optional[str] = resp.get("name")
+        self.url: typing.Optional[str] = resp.get("url")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.name:
             ret["name"] = self.name
@@ -672,12 +664,12 @@ class EmbedProvider(CopyableObject):
 
 class EmbedAuthor(CopyableObject):
     def __init__(self, resp):
-        self.name = resp.get("name")
-        self.url = resp.get("url")
-        self.icon_url = resp.get("icon_url")
-        self.proxy_icon_url = resp.get("proxy_icon_url")
+        self.name: typing.Optional[str] = resp.get("name")
+        self.url: typing.Optional[str] = resp.get("url")
+        self.icon_url: typing.Optional[str] = resp.get("icon_url")
+        self.proxy_icon_url: typing.Optional[str] = resp.get("proxy_icon_url")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {}
         if self.name:
             ret["name"] = self.name
@@ -697,11 +689,11 @@ class EmbedAuthor(CopyableObject):
 
 class EmbedFooter(CopyableObject):
     def __init__(self, resp):
-        self.text = resp["text"]
-        self.icon_url = resp.get("icon_url")
-        self.proxy_icon_url = resp.get("proxy_icon_url")
+        self.text: typing.Optional[str] = resp["text"]
+        self.icon_url: typing.Optional[str] = resp.get("icon_url")
+        self.proxy_icon_url: typing.Optional[str] = resp.get("proxy_icon_url")
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {"text": self.text}
         if self.icon_url:
             ret["icon_url"] = self.icon_url
@@ -717,11 +709,11 @@ class EmbedFooter(CopyableObject):
 
 class EmbedField(CopyableObject):
     def __init__(self, resp):
-        self.name = resp["name"]
-        self.value = resp["value"]
-        self.inline = resp.get("inline", True)
+        self.name: typing.Optional[str] = resp["name"]
+        self.value: typing.Optional[str] = resp["value"]
+        self.inline: typing.Optional[bool] = resp.get("inline", True)
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         return {"name": self.name, "value": self.value, "inline": self.inline}
 
     @classmethod
@@ -732,18 +724,18 @@ class EmbedField(CopyableObject):
 
 class Attachment:
     def __init__(self, client, resp):
-        self.client = client
-        self.id = Snowflake(resp["id"])
-        self.filename = resp["filename"]
-        self.content_type = resp.get("content_type")
-        self.size = resp["size"]
-        self.url = resp["url"]
-        self.proxy_url = resp["proxy_url"]
-        self.height = resp.get("height")
-        self.width = resp.get("width")
-        self.content = None  # Filled after download is called
+        self.client: APIClient = client
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.filename: str = resp["filename"]
+        self.content_type: typing.Optional[str] = resp.get("content_type")
+        self.size: int = resp["size"]
+        self.url: str = resp["url"]
+        self.proxy_url: str = resp["proxy_url"]
+        self.height: typing.Optional[int] = resp.get("height")
+        self.width: typing.Optional[int] = resp.get("width")
+        self.content: typing.Optional[bytes] = None  # Filled after download is called
 
-    def download(self):
+    def download(self) -> typing.Union[bytes, typing.Awaitable[bytes]]:
         dw = self.client.http.download(self.url)
         if inspect.isawaitable(dw):
             return self.__async_download(dw)
@@ -760,7 +752,7 @@ class Attachment:
         with open(str(target)+"/" if target and not str(target).endswith("/") else ""+self.filename, "wb") as f:
             f.write(self.content)
 
-    def to_dict(self):
+    def to_dict(self) -> dict:
         ret = {"id": str(self.id),
                "filename": self.filename,
                "size": self.size,
@@ -777,24 +769,24 @@ class Attachment:
 
 class ChannelMention:
     def __init__(self, resp):
-        self.id = Snowflake(resp["id"])
-        self.guild_id = Snowflake(resp["guild_id"])
-        self.type = ChannelTypes(resp["type"])
-        self.name = resp["name"]
+        self.id: Snowflake = Snowflake(resp["id"])
+        self.guild_id: Snowflake = Snowflake(resp["guild_id"])
+        self.type: ChannelTypes = ChannelTypes(resp["type"])
+        self.name: str = resp["name"]
 
 
 class AllowedMentions(CopyableObject):
     def __init__(self, *,
                  everyone: bool = False,
-                 users: typing.List[typing.Union[str, int, Snowflake]] = None,
-                 roles: typing.List[typing.Union[str, int, Snowflake]] = None,
+                 users: typing.List[Snowflake.TYPING] = None,
+                 roles: typing.List[Snowflake.TYPING] = None,
                  replied_user: bool = False):
         self.everyone: bool = everyone
-        self.users = users or []
-        self.roles = roles or []
+        self.users: typing.List[Snowflake.TYPING] = users or []
+        self.roles: typing.List[Snowflake.TYPING] = roles or []
         self.replied_user = replied_user
 
-    def to_dict(self, *, reply: bool = False):
+    def to_dict(self, *, reply: bool = False) -> dict:
         ret = {"parsed": []}
         if self.everyone:
             ret["parsed"].append("everyone")
@@ -811,6 +803,6 @@ class AllowedMentions(CopyableObject):
 
 class ListThreadsResponse:
     def __init__(self, client, resp):
-        self.threads = [Channel.create(client, x) for x in resp["threads"]]
-        self.members = [ThreadMember(client, x) for x in resp["members"]]
-        self.has_more = resp.get("has_more")
+        self.threads: typing.List[Channel] = [Channel.create(client, x) for x in resp["threads"]]
+        self.members: typing.List[ThreadMember] = [ThreadMember(client, x) for x in resp["members"]]
+        self.has_more: typing.Optional[bool] = resp.get("has_more")
