@@ -1,3 +1,5 @@
+import typing
+
 from .snowflake import Snowflake
 from ..base.model import DiscordObjectBase, FlagBase
 
@@ -39,9 +41,12 @@ class PermissionFlags(FlagBase):
     MANAGE_THREADS = 0x0400000000
     USE_PUBLIC_THREADS = 0x0800000000
     USE_PRIVATE_THREADS = 0x1000000000
+    USE_EXTERNAL_STICKERS = 0x2000000000
 
 
 class Role(DiscordObjectBase):
+    TYPING = typing.Union[int, str, Snowflake, "Role"]
+
     def __init__(self, client, resp, *, guild_id=None):
         super().__init__(client, resp)
         self.guild_id = Snowflake.optional(guild_id)  # This isn't actually in payload, but role is always created at the guild, so why not?
@@ -60,6 +65,12 @@ class Role(DiscordObjectBase):
         if self.client.has_cache:
             return self.client.get(self.guild_id, "guild")
 
+    def to_position_param(self, position: int = None):
+        body = {"id": str(self.id)}
+        if position is not None:
+            body["position"] = position
+        return body
+
 
 class RoleTags:
     def __init__(self, resp):
