@@ -53,11 +53,15 @@ class WebSocketClient:
             while not self._closed:
                 try:
                     resp = await self.receive()
+                except Ignore:
+                    continue
                 except WSClosing as ex:
                     self.logger.warning(f"Websocket is closing with code: {ex.code}")
-                    if ex.code is None and self.try_reconnect:
+                    if (ex.code in (1000, 1001) or ex.code is None) or self.try_reconnect:
                         self.logger.warning("Trying to reconnect...")
                         await self.reconnect(fresh=True)
+                    else:
+                        self.logger.error(f"Unexpected websocket disconnection; this client will now be terminated.")
                     break
                 if not resp:
                     # self.logger.warning("Empty response detected, ignoring.")
@@ -65,8 +69,8 @@ class WebSocketClient:
 
                     # Actually, we should terminate or we might have 500MB+ logger file ¯\_(ツ)_/¯
                     await self.close(1001)
-                    raise Exception("Uh oh, it looks like you just encountered some annoying issue. For the safety, this client will automatically closed.\n"
-                                    "If you have the logger set to DEBUG mode, please report this issue with 4~5 previous raw response lof to GitHub issue #1 or our support server.\n"
+                    raise Exception("Uh oh, it looks like you just encountered some annoying issue. For the safety, this client will be automatically closed.\n"
+                                    "If you have the logger set to DEBUG mode, please report this issue with 4~5 previous raw response log to GitHub issue #1 or our support server.\n"
                                     "Sorry for the inconvenience.")
                 self.logger.debug(f"Received `{gateway.Opcodes.as_string(resp.op)}` payload"
                                   f"{f' with event name `{resp.t}`' if resp.op == gateway.Opcodes.DISPATCH else ''}.")
@@ -117,7 +121,9 @@ class WebSocketClient:
             if res.s is not None:
                 self.seq = res.s
             return res
-        elif resp.type == aiohttp.WSMsgType.CLOSE:
+        elif resp.type == aiohttp.WSMsgType.CONTINUATION:
+            raise Ignore
+        elif resp.type in (aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.CLOSING):
             raise WSClosing(resp.data)
 
     async def reconnect(self, fresh: bool = False):
@@ -226,3 +232,7 @@ class WSClosing(Exception):
     """Internal exception class for controlling WS close event. You should not see this exception in normal situation."""
     def __init__(self, code):
         self.code = code
+
+
+class Ignore(Exception):
+    """Internal exception class for just ignoring the response. Used for ignoring unsupported response."""
